#ifndef _GFDTD_H
#define _GFDTD_H

/*
 * Copyright (C) 2002, Choi Ki-young, xenus@gnu.org
 */

#include "TMatrixD.h"


/*
 * physical constants (using GSL)
 */

/* speed of light in a vacuum */
#define LIGHT_SPEED GSL_CONST_MKS_SPEED_OF_LIGHT

/* m^2/s^2 */
#define LIGHT_SPEED_SQUARED (GSL_CONST_MKS_SPEED_OF_LIGHT*GSL_CONST_MKS_SPEED_OF_LIGHT)

/* permeability of free space */
#define MU_0 GSL_CONST_MKS_VACUUM_PERMEABILITY 
/* permittivity of free space */
#define EPSILON_0 GSL_CONST_MKS_VACUUM_PERMITTIVITY 

#define SH MU_0/EPSILON_0

#define SIGMA_0 0
#define RO_0 0


/* field system */

typedef struct Fields{
	
	/* TE mode */
	TMatrixD Ex;
	TMatrixD Ex1;
	TMatrixD Ex2;
	TMatrixD Ex3;
	TMatrixD Ex4;
	TMatrixD Ex5;
	TMatrixD Ex6;
	TMatrixD Ex7;
	TMatrixD Ex8;

	TMatrixD Ey;
	TMatrixD Ey1;
	TMatrixD Ey2;
	TMatrixD Ey3;
	TMatrixD Ey4;
	TMatrixD Ey5;
	TMatrixD Ey6;
	TMatrixD Ey7;
	TMatrixD Ey8;

	TMatrixD Hz;
	TMatrixD Hzx1;
	TMatrixD Hzy1;
	TMatrixD Hzx2;
	TMatrixD Hzy2;
	TMatrixD Hzx3;
	TMatrixD Hzy3;
	TMatrixD Hzx4;
	TMatrixD Hzy4;
	TMatrixD Hzx5;
	TMatrixD Hzy5;
	TMatrixD Hzx6;
	TMatrixD Hzy6;
	TMatrixD Hzx7;
	TMatrixD Hzy7;
	TMatrixD Hzx8;
	TMatrixD Hzy8;

	/* TM mode */
	TMatrixD Hx;
	TMatrixD Hx1;
	TMatrixD Hx2;
	TMatrixD Hx3;
	TMatrixD Hx4;
	TMatrixD Hx5;
	TMatrixD Hx6;
	TMatrixD Hx7;
	TMatrixD Hx8;

	TMatrixD Hy;
	TMatrixD Hy1;
	TMatrixD Hy2;
	TMatrixD Hy3;
	TMatrixD Hy4;
	TMatrixD Hy5;
	TMatrixD Hy6;
	TMatrixD Hy7;
	TMatrixD Hy8;

	TMatrixD Ez;
	TMatrixD Ezx1;
	TMatrixD Ezy1;
	TMatrixD Ezx2;
	TMatrixD Ezy2;
	TMatrixD Ezx3;
	TMatrixD Ezy3;
	TMatrixD Ezx4;
	TMatrixD Ezy4;
	TMatrixD Ezx5;
	TMatrixD Ezy5;
	TMatrixD Ezx6;
	TMatrixD Ezy6;
	TMatrixD Ezx7;
	TMatrixD Ezy7;
	TMatrixD Ezx8;
	TMatrixD Ezy8;

};

/* system coef. */
typedef struct Coef{

	// Free space
	TMatrixD Ca;
	TMatrixD Cb;
	TMatrixD Da;
	TMatrixD Db;

	// PML area 1
	TMatrixD Cax_1;
	TMatrixD Cbx_1;
	TMatrixD Dax_1;
	TMatrixD Dbx_1;
	TMatrixD Cay_1;
	TMatrixD Cby_1;
	TMatrixD Day_1;
	TMatrixD Dby_1;

	// PML area 2
	TMatrixD Cax_2;
	TMatrixD Cbx_2;
	TMatrixD Dax_2;
	TMatrixD Dbx_2;
	TMatrixD Cay_2;
	TMatrixD Cby_2;
	TMatrixD Day_2;
	TMatrixD Dby_2;

	// PML area 3
	TMatrixD Cax_3;
	TMatrixD Cbx_3;
	TMatrixD Dax_3;
	TMatrixD Dbx_3;
	TMatrixD Cay_3;
	TMatrixD Cby_3;
	TMatrixD Day_3;
	TMatrixD Dby_3;

	// PML area 4
	TMatrixD Cax_4;
	TMatrixD Cbx_4;
	TMatrixD Dax_4;
	TMatrixD Dbx_4;
	TMatrixD Cay_4;
	TMatrixD Cby_4;
	TMatrixD Day_4;
	TMatrixD Dby_4;

	// PML area 5
	TMatrixD Ca_5;
	TMatrixD Cb_5;
	TMatrixD Da_5;
	TMatrixD Db_5;

	// PML area 6
	TMatrixD Ca_6;
	TMatrixD Cb_6;
	TMatrixD Da_6;
	TMatrixD Db_6;

	// PML area 7
	TMatrixD Ca_7;
	TMatrixD Cb_7;
	TMatrixD Da_7;
	TMatrixD Db_7;

	// PML area 8
	TMatrixD Ca_8;
	TMatrixD Cb_8;
	TMatrixD Da_8;
	TMatrixD Db_8;

};

/*
 * sub functions
 */

/* init. field */
void init(Fields *fields, Coef *coef, int *MAXTIMESTEP, int *X_SIZE, int *Y_SIZE, int *PML_DEPTH, double *FREQ, double *dT, char *mode); 
/* update equation */
void update_fields(Fields *fields, Coef *coef, int iteration, int X_SIZE, int Y_SIZE, int PML_DEPTH, double FREQ, double dT, char *mode); 
/* clear field */
void clear_fields(Fields *fields, char *mode);

#endif _GFDTD_H
