#include <stdio.h>
#include <math.h>
#include <gsl/gsl_const_mks.h>
#include "gfdtd.h"


/*
 * numbers are a number of PML area *
 *
 
 111115555555555555555522222
 111115555555555555555522222
 111115555555555555555522222
 88888                 66666
 88888                 66666
 88888                 66666
 88888    Free Space   66666
 88888                 66666
 88888                 66666
 88888                 66666
 88888                 66666
 444447777777777777777733333
 444447777777777777777733333
 444447777777777777777733333

 */

void init(Fields *fields, Coef *coef, int *MAXTIMESTEP, int *X, int *Y, int *PML, double *FREQ, double *DT, char *mode)
{
	FILE *input;
	FILE *fpt;
	int i,j,k,l;
	int x,y;
	int X_SIZE;
	int Y_SIZE;
	int PML_DEPTH;
	double sigma_m;
	double LAMBDA;
	double CELL_SIZE;
	double epsilon;
	double dT;
	short int system[1000][1000];


	printf("\n\n = System initialization start.\n\n\n");
	input = fopen("input.dat", "r");

	epsilon = 11.4*EPSILON_0;

	fscanf(input,"%s",mode);
	fscanf(input,"%lf",&sigma_m);
	fscanf(input,"%d",MAXTIMESTEP);
	fscanf(input,"%d",&X_SIZE);
	fscanf(input,"%d",&Y_SIZE);
	fscanf(input,"%d",&PML_DEPTH);
	fscanf(input,"%lf",FREQ);
	fscanf(input,"%lf",&CELL_SIZE);

	fclose(input);

	LAMBDA = 2*M_PI*(LIGHT_SPEED / *FREQ);
	CELL_SIZE = CELL_SIZE*0.000000001;
	dT = CELL_SIZE/(sqrt(2)*LIGHT_SPEED);
	*DT = dT;

        for(i=0;i<1000;i++)
                for(j=0;j<1000;j++)
                        system[i][j] = 0;

	/*
        for(i=50;i<400;i++)
                for(j=0;j<400;j++)
                        system[i][j] = 1;
	*/

	for(i=50;i<999;i+=20)
		for(j=10;j<510;j+=20)
	//	for(j=10;j<400;j+=20)
			for(k=0;k<1000;k++)
				for(l=0;l<1000;l++)
					if(sqrt((k-i)*(k-i)+(l-j)*(l-j)) <= 4.0)
						system[k][l] = 1;
	
	for(i=50;i<999;i+=20)
		for(j=530;j<1000;j+=20)
			for(k=0;k<1000;k++)
				for(l=0;l<1000;l++)
					if(sqrt((k-i)*(k-i)+(l-j)*(l-j)) <= 4.0)
						system[k][l] = 1;

	fpt = fopen("system.dat", "w");
        for(j=0;j<1000;j++)
        {
        	for(i=0;i<1000;i++)
                        fprintf(fpt,"%d ",system[i][j]);
                fprintf(fpt,"\n");
        }
        fclose(fpt);

	printf("sigma_m : %f\n",sigma_m);
	printf("MAXTIMESTEP : %d\n",*MAXTIMESTEP);
	printf("X_SIZE : %d\n",X_SIZE);
	printf("Y_SIZE : %d\n",Y_SIZE);
	printf("PML_DEPTH : %d\n",PML_DEPTH);
	printf("CELL_SIZE : %e\n",CELL_SIZE);
	printf("delta T : %e\n", dT);
	printf("LIGHT_SPEED : %e\n", LIGHT_SPEED);
	printf("LAMBDA : %e\n", LAMBDA);
	printf("FREQ : %e\n",*FREQ);

	*X = X_SIZE;
	*Y = Y_SIZE;
	*PML = PML_DEPTH;


	double * sigma_p = new double [PML_DEPTH];

	for(i=0;i<PML_DEPTH;i++)
		sigma_p[i] = sigma_m*(i+1)*(i+1)*(i+1)/(PML_DEPTH*PML_DEPTH*PML_DEPTH);

	sigma_p[0] = ((sigma_p[0] - 0)/2 + sigma_p[0] + (sigma_p[0] + sigma_p[1])/2)/3;
	for(i=1;i<PML_DEPTH-1;i++)
		sigma_p[i] = ((sigma_p[i-1]+sigma_p[i])/2 + sigma_p[i] + (sigma_p[i] + sigma_p[i+1])/2)/3;

	/* Memory allocation for system coef. */

	// Free Space
	coef->Ca.ResizeTo(X_SIZE, Y_SIZE);
	coef->Cb.ResizeTo(X_SIZE, Y_SIZE);
	coef->Da.ResizeTo(X_SIZE, Y_SIZE);
	coef->Db.ResizeTo(X_SIZE, Y_SIZE);

	// PML area 1
	coef->Cax_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cbx_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dax_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dbx_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cay_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cby_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Day_1.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dby_1.ResizeTo(PML_DEPTH, PML_DEPTH);

	// PML area 2
	coef->Cax_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cbx_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dax_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dbx_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cay_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cby_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Day_2.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dby_2.ResizeTo(PML_DEPTH, PML_DEPTH);

	// PML area 3
	coef->Cax_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cbx_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dax_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dbx_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cay_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cby_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Day_3.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dby_3.ResizeTo(PML_DEPTH, PML_DEPTH);

	// PML area 4
	coef->Cax_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cbx_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dax_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dbx_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cay_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Cby_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Day_4.ResizeTo(PML_DEPTH, PML_DEPTH);
	coef->Dby_4.ResizeTo(PML_DEPTH, PML_DEPTH);

	// PML area 5
	coef->Ca_5.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Cb_5.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Da_5.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Db_5.ResizeTo(X_SIZE,PML_DEPTH);

	// PML area 6
	coef->Ca_6.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Cb_6.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Da_6.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Db_6.ResizeTo(PML_DEPTH,Y_SIZE);

	// PML area 7
	coef->Ca_7.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Cb_7.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Da_7.ResizeTo(X_SIZE,PML_DEPTH);
	coef->Db_7.ResizeTo(X_SIZE,PML_DEPTH);

	// PML area 8
	coef->Ca_8.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Cb_8.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Da_8.ResizeTo(PML_DEPTH,Y_SIZE);
	coef->Db_8.ResizeTo(PML_DEPTH,Y_SIZE);

	/* TE mode */

	if(strcmp(mode,"TE")==0)
	{
		printf("\n\nTE mode initialized!\n\n");
		/* Memory allocation for fields of free space */
		fields->Ex.ResizeTo(X_SIZE ,Y_SIZE);
		fields->Ey.ResizeTo(X_SIZE ,Y_SIZE);
		fields->Hz.ResizeTo(X_SIZE ,Y_SIZE);


		/* Memory allocation for fields of PML */

		/* area 1 */

		fields->Ex1.ResizeTo(PML_DEPTH, PML_DEPTH+1);
		fields->Ey1.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzx1.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzy1.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 2 */

		fields->Ex2.ResizeTo(PML_DEPTH, PML_DEPTH+1);
		fields->Ey2.ResizeTo(PML_DEPTH+1, PML_DEPTH);
		fields->Hzx2.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzy2.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 3 */

		fields->Ex3.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ey3.ResizeTo(PML_DEPTH+1, PML_DEPTH);
		fields->Hzx3.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzy3.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 4 */

		fields->Ex4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ey4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzx4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hzy4.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 5 */

		fields->Ex5.ResizeTo(X_SIZE, PML_DEPTH+1);
		fields->Ey5.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Hzx5.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Hzy5.ResizeTo(X_SIZE, PML_DEPTH);

		/* area 6 */

		fields->Ex6.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Ey6.ResizeTo(PML_DEPTH+1, Y_SIZE);
		fields->Hzx6.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Hzy6.ResizeTo(PML_DEPTH, Y_SIZE);

		/* area 7 */

		fields->Ex7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Ey7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Hzx7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Hzy7.ResizeTo(X_SIZE, PML_DEPTH);

		/* area 8 */
	
		fields->Ex8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Ey8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Hzx8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Hzy8.ResizeTo(PML_DEPTH, Y_SIZE);

	
		/* init Hz field */
		fields->Hz.Zero();
		fields->Hzx1.Zero();
		fields->Hzy1.Zero();
		fields->Hzx2.Zero();
		fields->Hzy2.Zero();
		fields->Hzx3.Zero();
		fields->Hzy3.Zero();
		fields->Hzx4.Zero();
		fields->Hzy4.Zero();
		fields->Hzx5.Zero();
		fields->Hzy5.Zero();
		fields->Hzx6.Zero();
		fields->Hzy6.Zero();
		fields->Hzx7.Zero();
		fields->Hzy7.Zero();
		fields->Hzx8.Zero();
		fields->Hzy8.Zero();

		/* init Ex field */
		fields->Ex.Zero();
		fields->Ex1.Zero();
		fields->Ex2.Zero();
		fields->Ex3.Zero();
		fields->Ex4.Zero();
		fields->Ex5.Zero();
		fields->Ex6.Zero();
		fields->Ex7.Zero();
		fields->Ex8.Zero();

		/* init Ey field */
		fields->Ey.Zero();
		fields->Ey1.Zero();
		fields->Ey2.Zero();
		fields->Ey3.Zero();
		fields->Ey4.Zero();
		fields->Ey5.Zero();
		fields->Ey6.Zero();
		fields->Ey7.Zero();
		fields->Ey8.Zero();

	}

	/* TM mode */

	if(strcmp(mode,"TM")==0)
	{
		printf("\n\nTM mode initialized!\n\n");
		/* Memory allocation for fields of free space */
		fields->Hx.ResizeTo(X_SIZE ,Y_SIZE);
		fields->Hy.ResizeTo(X_SIZE ,Y_SIZE);
		fields->Ez.ResizeTo(X_SIZE ,Y_SIZE);


		/* Memory allocation for fields of PML */

		/* area 1 */

		fields->Hx1.ResizeTo(PML_DEPTH, PML_DEPTH+1);
		fields->Hy1.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezx1.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezy1.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 2 */

		fields->Hx2.ResizeTo(PML_DEPTH, PML_DEPTH+1);
		fields->Hy2.ResizeTo(PML_DEPTH+1, PML_DEPTH);
		fields->Ezx2.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezy2.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 3 */

		fields->Hx3.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hy3.ResizeTo(PML_DEPTH+1, PML_DEPTH);
		fields->Ezx3.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezy3.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 4 */

		fields->Hx4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Hy4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezx4.ResizeTo(PML_DEPTH, PML_DEPTH);
		fields->Ezy4.ResizeTo(PML_DEPTH, PML_DEPTH);

		/* area 5 */

		fields->Hx5.ResizeTo(X_SIZE, PML_DEPTH+1);
		fields->Hy5.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Ezx5.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Ezy5.ResizeTo(X_SIZE, PML_DEPTH);

		/* area 6 */

		fields->Hx6.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Hy6.ResizeTo(PML_DEPTH+1, Y_SIZE);
		fields->Ezx6.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Ezy6.ResizeTo(PML_DEPTH, Y_SIZE);

		/* area 7 */

		fields->Hx7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Hy7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Ezx7.ResizeTo(X_SIZE, PML_DEPTH);
		fields->Ezy7.ResizeTo(X_SIZE, PML_DEPTH);

		/* area 8 */
	
		fields->Hx8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Hy8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Ezx8.ResizeTo(PML_DEPTH, Y_SIZE);
		fields->Ezy8.ResizeTo(PML_DEPTH, Y_SIZE);

	
		/* init Ez field */
		fields->Ez.Zero();
		fields->Ezx1.Zero();
		fields->Ezy1.Zero();
		fields->Ezx2.Zero();
		fields->Ezy2.Zero();
		fields->Ezx3.Zero();
		fields->Ezy3.Zero();
		fields->Ezx4.Zero();
		fields->Ezy4.Zero();
		fields->Ezx5.Zero();
		fields->Ezy5.Zero();
		fields->Ezx6.Zero();
		fields->Ezy6.Zero();
		fields->Ezx7.Zero();
		fields->Ezy7.Zero();
		fields->Ezx8.Zero();
		fields->Ezy8.Zero();

		/* init Hx field */
		fields->Hx.Zero();
		fields->Hx1.Zero();
		fields->Hx2.Zero();
		fields->Hx3.Zero();
		fields->Hx4.Zero();
		fields->Hx5.Zero();
		fields->Hx6.Zero();
		fields->Hx7.Zero();
		fields->Hx8.Zero();

		/* init Hy field */
		fields->Hy.Zero();
		fields->Hy1.Zero();
		fields->Hy2.Zero();
		fields->Hy3.Zero();
		fields->Hy4.Zero();
		fields->Hy5.Zero();
		fields->Hy6.Zero();
		fields->Hy7.Zero();
		fields->Hy8.Zero();

	}


	/* init system coefficients(free space) */

	for(i=0;i<X_SIZE;i++)
		for(j=0;j<Y_SIZE;j++)
		{
			if(system[i][j] == 0)
			{

				coef->Ca(i,j) = (1-SIGMA_0*dT/(2*EPSILON_0))/(1+SIGMA_0*dT/(2*EPSILON_0));
				coef->Cb(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+SIGMA_0*dT/(2*EPSILON_0));
				coef->Da(i,j) = (1-SIGMA_0*dT/(2*MU_0))/(1+SIGMA_0*dT/(2*MU_0));
				coef->Db(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+RO_0*dT/(2*MU_0));
			}

			else
			{
				coef->Ca(i,j) = (1-SIGMA_0*dT/(2*epsilon))/(1+SIGMA_0*dT/(2*epsilon));
				coef->Cb(i,j) = sqrt(epsilon/MU_0)*(dT/(epsilon*CELL_SIZE))/(1+SIGMA_0*dT/(2*epsilon));
				coef->Da(i,j) = (1-SIGMA_0*dT/(2*MU_0))/(1+SIGMA_0*dT/(2*MU_0));
				coef->Db(i,j) = sqrt(MU_0/(epsilon))*(dT/(MU_0*CELL_SIZE))/(1+RO_0*dT/(2*MU_0));
			}
		}/* end of for(j=0;j<Y_SIZE;j++) */

	/* init system coefficients(PML) 
	   assume, n = 2, sigma_m = 1480000 */

	/* PML 1~4 */
	for(i=0;i<PML_DEPTH;i++)
		for(j=0;j<PML_DEPTH;j++)
		{
			coef->Cax_1(i,j) = (1-sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Cbx_1(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Dax_1(i,j) = (1-SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));
			coef->Dbx_1(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));

			coef->Cax_2(i,j) = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Cbx_2(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Dax_2(i,j) = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
			coef->Dbx_2(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[i]*dT/(2*MU_0));

			coef->Cax_3(i,j) = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Cbx_3(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Dax_3(i,j) = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
			coef->Dbx_3(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[i]*dT/(2*MU_0));

			coef->Cax_4(i,j) = (1-sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Cbx_4(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Dax_4(i,j) = (1-SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));
			coef->Dbx_4(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));

			coef->Cay_1(i,j) = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Cby_1(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Day_1(i,j) = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
			coef->Dby_1(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[j]*dT/(2*MU_0));

			coef->Cay_2(i,j) = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Cby_2(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Day_2(i,j) = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
			coef->Dby_2(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[j]*dT/(2*MU_0));

			coef->Cay_3(i,j) = (1-sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Cby_3(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Day_3(i,j) = (1-SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));
			coef->Dby_3(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));

			coef->Cay_4(i,j) = (1-sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Cby_4(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Day_4(i,j) = (1-SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));
			coef->Dby_4(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));
		}

	/* PML 5,7 */
	for(i=0;i<X_SIZE;i++)
		for(j=0;j<PML_DEPTH;j++)
		{
			coef->Ca_5(i,j) = (1-sigma_p[j]*dT/(2*EPSILON_0))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Cb_5(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[j]*dT/(2*EPSILON_0));
			coef->Da_5(i,j) = (1-SH*sigma_p[j]*dT/(2*MU_0))/(1+SH*sigma_p[j]*dT/(2*MU_0));
			coef->Db_5(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[j]*dT/(2*MU_0));

			coef->Ca_7(i,j) = (1-sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Cb_7(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-j]*dT/(2*EPSILON_0));
			coef->Da_7(i,j) = (1-SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));
			coef->Db_7(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-j]*dT/(2*MU_0));
		}

	/* PML 6,8 */
	for(i=0;i<PML_DEPTH;i++)
		for(j=0;j<Y_SIZE;j++)
		{
			coef->Ca_6(i,j) = (1-sigma_p[i]*dT/(2*EPSILON_0))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Cb_6(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[i]*dT/(2*EPSILON_0));
			coef->Da_6(i,j) = (1-SH*sigma_p[i]*dT/(2*MU_0))/(1+SH*sigma_p[i]*dT/(2*MU_0));
			coef->Db_6(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[i]*dT/(2*MU_0));

			coef->Ca_8(i,j) = (1-sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Cb_8(i,j) = sqrt(EPSILON_0/MU_0)*(dT/(EPSILON_0*CELL_SIZE))/(1+sigma_p[PML_DEPTH-1-i]*dT/(2*EPSILON_0));
			coef->Da_8(i,j) = (1-SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));
			coef->Db_8(i,j) = sqrt(MU_0/EPSILON_0)*(dT/(MU_0*CELL_SIZE))/(1+SH*sigma_p[PML_DEPTH-1-i]*dT/(2*MU_0));
		}

	delete [] sigma_p;

	/* System construct */


	printf("\n\n = System initialization end\n\n\n");


}
