#include <math.h>
#include <gsl/gsl_const_mks.h>
#include "gfdtd.h"

/*
 * Fields coefficients
 */

void update_fields(Fields *fields, Coef *coef, int iteration, int X_SIZE, int Y_SIZE, int PML_DEPTH, double FREQ, double dT, char *mode)
{
	int i,j;
	double source;

	/*********** 
	 * TE mode *
	 ***********/

	if(strcmp(mode,"TE")==0)
	{
		/**************************
		 *    Hz, Hxz, Hxy field  *
		 **************************/

		//source = 10*exp(-pow((20-iteration)/2,2.0));
		source = sin(2*M_PI*iteration*dT*FREQ);
		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==X_SIZE-1 && j==Y_SIZE-1)
				{
					fields->Hz(i,j) = coef->Da(i,j)*fields->Hz(i,j) + coef->Db(i,j)*(fields->Ex(i,0) - fields->Ex(i,Y_SIZE-1) - fields->Ey6(0,j) + fields->Ey(X_SIZE-1,j));
					if(i==200 && j==200)
						fields->Hz(i,j) += source;
				}
				else if(i==X_SIZE-1)
				{
					fields->Hz(i,j) = coef->Da(i,j)*fields->Hz(i,j) + coef->Db(i,j)*(fields->Ex(i,j+1) - fields->Ex(i,j) - fields->Ey6(0,j) + fields->Ey(X_SIZE-1,j));
					if(i==200 && j==200)
						fields->Hz(i,j) += source;
				}
				else if(j==Y_SIZE-1)
				{
					fields->Hz(i,j) = coef->Da(i,j)*fields->Hz(i,j) + coef->Db(i,j)*(fields->Ex5(i,0) - fields->Ex(i,j) - fields->Ey(i+1,j) + fields->Ey(i,j));
					if(i==200 && j==200)
						fields->Hz(i,j) += source;
				}
				else
				{
					fields->Hz(i,j) = coef->Da(i,j)*fields->Hz(i,j) + coef->Db(i,j)*(fields->Ex(i,j+1) - fields->Ex(i,j) - fields->Ey(i+1,j) + fields->Ey(i,j));
					if(i==200 && j==200)
						fields->Hz(i,j) += source;
				}
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==PML_DEPTH-1)
				{
					fields->Hzx1(i,j) = coef->Dax_1(i,j)*fields->Hzx1(i,j) - coef->Dbx_1(i,j)*(fields->Ey1(1,j) - fields->Ey1(i,j));
					fields->Hzy1(i,j) = coef->Day_1(i,j)*fields->Hzy1(i,j) + coef->Dby_1(i,j)*(fields->Ex5(0,j+1) - fields->Ex1(i,j));
				}
				else
				{
					fields->Hzx1(i,j) = coef->Dax_1(i,j)*fields->Hzx1(i,j) - coef->Dbx_1(i,j)*(fields->Ey1(i+1,j) - fields->Ey1(i,j));
					fields->Hzy1(i,j) = coef->Day_1(i,j)*fields->Hzy1(i,j) + coef->Dby_1(i,j)*(fields->Ex1(i,j+1) - fields->Ex1(i,j));
				}
			}
	
		// PML Area 2 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				fields->Hzx2(i,j) = coef->Dax_2(i,j)*fields->Hzx2(i,j) - coef->Dbx_2(i,j)*(fields->Ey2(i+1,j) - fields->Ey2(i,j));
				fields->Hzy2(i,j) = coef->Day_2(i,j)*fields->Hzy2(i,j) + coef->Dby_2(i,j)*(fields->Ex2(i,j+1) - fields->Ex2(i,j));
			}
	
		// PML Area 3 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=1;j<PML_DEPTH-1;j++)
			{
				if(j==PML_DEPTH-2)
				{
					fields->Hzx3(i,j) = coef->Dax_3(i,j)*fields->Hzx3(i,j) - coef->Dbx_3(i,j)*(fields->Ey3(i+1,j) - fields->Ey3(i,j));
					fields->Hzy3(i,j) = coef->Day_3(i,j)*fields->Hzy3(i,j) + coef->Dby_3(i,j)*(fields->Ex6(i,0) - fields->Ex3(i,j));
				}
				else
				{
					fields->Hzx3(i,j) = coef->Dax_3(i,j)*fields->Hzx3(i,j) - coef->Dbx_3(i,j)*(fields->Ey3(i+1,j) - fields->Ey3(i,j));
					fields->Hzy3(i,j) = coef->Day_3(i,j)*fields->Hzy3(i,j) + coef->Dby_3(i,j)*(fields->Ex3(i,j+1) - fields->Ex3(i,j));
				}
			}

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==PML_DEPTH-1 && j==PML_DEPTH-1)
				{
					fields->Hzx4(i,j) = coef->Dax_4(i,j)*fields->Hzx4(i,j) - coef->Dbx_4(i,j)*(fields->Ey7(0,j) - fields->Ey4(i,j));
					fields->Hzy4(i,j) = coef->Day_4(i,j)*fields->Hzy4(i,j) + coef->Dby_4(i,j)*(fields->Ex8(i,0) - fields->Ex4(i,j));
				}
				else if(i==PML_DEPTH-1)
				{
					fields->Hzx4(i,j) = coef->Dax_4(i,j)*fields->Hzx4(i,j) - coef->Dbx_4(i,j)*(fields->Ey7(0,j) - fields->Ey4(i,j));
					fields->Hzy4(i,j) = coef->Day_4(i,j)*fields->Hzy4(i,j) + coef->Dby_4(i,j)*(fields->Ex4(i,j+1) - fields->Ex4(i,j));
				}
				else if(j==PML_DEPTH-1)
				{
					fields->Hzx4(i,j) = coef->Dax_4(i,j)*fields->Hzx4(i,j) - coef->Dbx_4(i,j)*(fields->Ey4(i+1,j) - fields->Ey4(i,j));
					fields->Hzy4(i,j) = coef->Day_4(i,j)*fields->Hzy4(i,j) + coef->Dby_4(i,j)*(fields->Ex8(i,0) - fields->Ex4(i,j));
				}
				else
				{
					fields->Hzx4(i,j) = coef->Dax_4(i,j)*fields->Hzx4(i,j) - coef->Dbx_4(i,j)*(fields->Ey4(i+1,j) - fields->Ey4(i,j));
					fields->Hzy4(i,j) = coef->Day_4(i,j)*fields->Hzy4(i,j) + coef->Dby_4(i,j)*(fields->Ex4(i,j+1) - fields->Ex4(i,j));
				}
			}

		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==X_SIZE-1)
				{
					fields->Hzx5(i,j) = coef->Da_5(i,j)*fields->Hzx5(i,j) - coef->Db_5(i,j)*(fields->Ey2(0,j) - fields->Ey5(i,j));
					fields->Hzy5(i,j) = coef->Da_5(i,j)*fields->Hzy5(i,j) + coef->Db_5(i,j)*(fields->Ex5(i,j+1) - fields->Ex5(i,j));
				}
				else
				{
					fields->Hzx5(i,j) = coef->Da_5(i,j)*fields->Hzx5(i,j) - coef->Db_5(i,j)*(fields->Ey5(i+1,j) - fields->Ey5(i,j));
					fields->Hzy5(i,j) = coef->Da_5(i,j)*fields->Hzy5(i,j) + coef->Db_5(i,j)*(fields->Ex5(i,j+1) - fields->Ex5(i,j));
				}
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==Y_SIZE-1)
				{
					fields->Hzx6(i,j) = coef->Da_6(i,j)*fields->Hzx6(i,j) - coef->Db_6(i,j)*(fields->Ey6(i+1,j) - fields->Ey6(i,j));
					fields->Hzy6(i,j) = coef->Da_6(i,j)*fields->Hzy6(i,j) + coef->Db_6(i,j)*(fields->Ex2(i,0) - fields->Ex6(i,j));
				}
				else
				{
					fields->Hzx6(i,j) = coef->Da_6(i,j)*fields->Hzx6(i,j) - coef->Db_6(i,j)*(fields->Ey6(i+1,j) - fields->Ey6(i,j));
					fields->Hzy6(i,j) = coef->Da_6(i,j)*fields->Hzy6(i,j) + coef->Db_6(i,j)*(fields->Ex6(i,j+1) - fields->Ex6(i,j));
				}
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==X_SIZE-1 && j==PML_DEPTH-1)
				{
					fields->Hzx7(i,j) = coef->Da_7(i,j)*fields->Hzx7(i,j) - coef->Db_7(i,j)*(fields->Ey3(0,j) - fields->Ey7(i,j));
					fields->Hzy7(i,j) = coef->Da_7(i,j)*fields->Hzy7(i,j) + coef->Db_7(i,j)*(fields->Ex(i,0) - fields->Ex7(i,j));
				}
				else if(i==X_SIZE-1)
				{
					fields->Hzx7(i,j) = coef->Da_7(i,j)*fields->Hzx7(i,j) - coef->Db_7(i,j)*(fields->Ey3(0,j) - fields->Ey7(i,j));
					fields->Hzy7(i,j) = coef->Da_7(i,j)*fields->Hzy7(i,j) + coef->Db_7(i,j)*(fields->Ex7(i,j+1) - fields->Ex7(i,j));
				}
				else if(j==PML_DEPTH-1)
				{
					fields->Hzx7(i,j) = coef->Da_7(i,j)*fields->Hzx7(i,j) - coef->Db_7(i,j)*(fields->Ey7(i+1,j) - fields->Ey7(i,j));
					fields->Hzy7(i,j) = coef->Da_7(i,j)*fields->Hzy7(i,j) + coef->Db_7(i,j)*(fields->Ex(i,0) - fields->Ex7(i,j));
				}
				else
				{
					fields->Hzx7(i,j) = coef->Da_7(i,j)*fields->Hzx7(i,j) - coef->Db_7(i,j)*(fields->Ey7(i+1,j) - fields->Ey7(i,j));
					fields->Hzy7(i,j) = coef->Da_7(i,j)*fields->Hzy7(i,j) + coef->Db_7(i,j)*(fields->Ex7(i,j+1) - fields->Ex7(i,j));
				}
			}

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==PML_DEPTH-1 && j==Y_SIZE-1)
				{
					fields->Hzx8(i,j) = coef->Da_8(i,j)*fields->Hzx8(i,j) - coef->Db_8(i,j)*(fields->Ey(0,j) - fields->Ey8(i,j));
					fields->Hzy8(i,j) = coef->Da_8(i,j)*fields->Hzy8(i,j) + coef->Db_8(i,j)*(fields->Ex1(i,0) - fields->Ex8(i,j));
				}
				else if(i==PML_DEPTH-1)
				{
					fields->Hzx8(i,j) = coef->Da_8(i,j)*fields->Hzx8(i,j) - coef->Db_8(i,j)*(fields->Ey(0,j) - fields->Ey8(i,j));
					fields->Hzy8(i,j) = coef->Da_8(i,j)*fields->Hzy8(i,j) + coef->Db_8(i,j)*(fields->Ex8(i,j+1) - fields->Ex8(i,j));
				}
				else if(j==Y_SIZE-1)
				{
					fields->Hzx8(i,j) = coef->Da_8(i,j)*fields->Hzx8(i,j) - coef->Db_8(i,j)*(fields->Ey8(i+1,j) - fields->Ey8(i,j));
					fields->Hzy8(i,j) = coef->Da_8(i,j)*fields->Hzy8(i,j) + coef->Db_8(i,j)*(fields->Ex1(i,0) - fields->Ex8(i,j));
				}
				else
				{
					fields->Hzx8(i,j) = coef->Da_8(i,j)*fields->Hzx8(i,j) - coef->Db_8(i,j)*(fields->Ey8(i+1,j) - fields->Ey8(i,j));
					fields->Hzy8(i,j) = coef->Da_8(i,j)*fields->Hzy8(i,j) + coef->Db_8(i,j)*(fields->Ex8(i,j+1) - fields->Ex8(i,j));
				}
			}

		/**************************
		 *       Ex field         *
		 **************************/

		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Ex(i,j) = coef->Ca_7(i,PML_DEPTH-1)*fields->Ex(i,j) + coef->Cb_7(i,PML_DEPTH-1)*(fields->Hz(i,j) - ( fields->Hzx7(i,PML_DEPTH-1)+fields->Hzy7(i,PML_DEPTH-1) ));
				else
					fields->Ex(i,j) = coef->Ca(i,j)*fields->Ex(i,j) + coef->Cb(i,j)*(fields->Hz(i,j) - fields->Hz(i,j-1));
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Ex1(i,j) = coef->Cay_1(i,j)*fields->Ex1(i,j) + coef->Cby_1(i,j)*( ( fields->Hzx1(i,j)+fields->Hzy1(i,j) )-(fields->Hzx8(i,Y_SIZE-1)+fields->Hzy8(i,Y_SIZE-1)) );
				else
					fields->Ex1(i,j) = coef->Cay_1(i,j)*fields->Ex1(i,j) + coef->Cby_1(i,j)*( ( fields->Hzx1(i,j)+fields->Hzy1(i,j) )-(fields->Hzx1(i,j-1)+fields->Hzy1(i,j-1)) );
			}

		// PML Area 2 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Ex2(i,j) = coef->Cay_2(i,j)*fields->Ex2(i,j) + coef->Cby_2(i,j)*( ( fields->Hzx2(i,j)+fields->Hzy2(i,j) )-(fields->Hzx6(i,Y_SIZE-1)+fields->Hzy6(i,Y_SIZE-1)) );
				else
					fields->Ex2(i,j) = coef->Cay_2(i,j)*fields->Ex2(i,j) + coef->Cby_2(i,j)*( ( fields->Hzx2(i,j)+fields->Hzy2(i,j) )-(fields->Hzx2(i,j-1)+fields->Hzy2(i,j-1)) );
			}

		// PML Area 3 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Ex3(i,j) = coef->Cay_3(i,j)*fields->Ex3(i,j) + coef->Cby_3(i,j)*( ( fields->Hzx3(i,j)+fields->Hzy3(i,j) )-(fields->Hzx3(i,j-1)+fields->Hzy3(i,j-1)) );

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Ex4(i,j) = coef->Cay_4(i,j)*fields->Ex4(i,j) + coef->Cby_4(i,j)*( ( fields->Hzx4(i,j)+fields->Hzy4(i,j) )-(fields->Hzx4(i,j-1)+fields->Hzy4(i,j-1)) );

		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Ex5(i,j) = coef->Ca_5(i,j)*fields->Ex5(i,j) + coef->Cb_5(i,j)*( ( fields->Hzx5(i,j)+fields->Hzy5(i,j) )-fields->Hz(i,Y_SIZE-1) );
				else
					fields->Ex5(i,j) = coef->Ca_5(i,j)*fields->Ex5(i,j) + coef->Cb_5(i,j)*( ( fields->Hzx5(i,j)+fields->Hzy5(i,j) )-(fields->Hzx5(i,j-1)+fields->Hzy5(i,j-1)) );
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Ex6(i,j) = coef->Ca_6(i,j)*fields->Ex6(i,j) + coef->Cb_6(i,j)*( ( fields->Hzx6(i,j)+fields->Hzy6(i,j) )-(fields->Hzx3(i,PML_DEPTH-1)+fields->Hzy3(i,PML_DEPTH-1)) );
				else
					fields->Ex6(i,j) = coef->Ca_6(i,j)*fields->Ex6(i,j) + coef->Cb_6(i,j)*( ( fields->Hzx6(i,j)+fields->Hzy6(i,j) )-(fields->Hzx6(i,j-1)+fields->Hzy6(i,j-1)) );
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Ex7(i,j) = coef->Ca_7(i,j)*fields->Ex7(i,j) + coef->Cb_7(i,j)*( ( fields->Hzx7(i,j)+fields->Hzy7(i,j) )-(fields->Hzx7(i,j-1)+fields->Hzy7(i,j-1)) );

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Ex8(i,j) = coef->Ca_8(i,j)*fields->Ex8(i,j) + coef->Cb_8(i,j)*( ( fields->Hzx8(i,j)+fields->Hzy8(i,j) )-(fields->Hzx4(i,PML_DEPTH-1)+fields->Hzy4(i,PML_DEPTH-1)) );
				else
					fields->Ex8(i,j) = coef->Ca_8(i,j)*fields->Ex8(i,j) + coef->Cb_8(i,j)*( ( fields->Hzx8(i,j)+fields->Hzy8(i,j) )-(fields->Hzx8(i,j-1)+fields->Hzy8(i,j-1)) );
			}



		/**************************
		 *       Ey field         *
		 **************************/
	
		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==0)
					fields->Ey(i,j) = coef->Ca_8(PML_DEPTH-1,j)*fields->Ey(i,j) - coef->Cb_8(PML_DEPTH-1,j)*( fields->Hz(i,j) - (fields->Hzx8(PML_DEPTH-1,j)+fields->Hzy8(PML_DEPTH-1,j) ));
				else
					fields->Ey(i,j) = coef->Ca(i,j)*fields->Ey(i,j) - coef->Cb(i,j)*(fields->Hz(i,j) - fields->Hz(i-1,j));
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
				fields->Ey1(i,j) = coef->Cax_1(i,j)*fields->Ey1(i,j) - coef->Cbx_1(i,j)*( (fields->Hzx1(i,j)+fields->Hzy1(i,j) ) - ( fields->Hzx1(i-1,j)+fields->Hzy1(i-1,j)) );

		// PML Area 2 
		for(i=0;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Ey2(i,j) = coef->Cax_2(i,j)*fields->Ey2(i,j) - coef->Cbx_2(i,j)*( (fields->Hzx2(i,j)+fields->Hzy2(i,j) ) - (fields->Hzx5(X_SIZE-1,j)+fields->Hzy5(X_SIZE-1,j) ) );
				else
					fields->Ey2(i,j) = coef->Cax_2(i,j)*fields->Ey2(i,j) - coef->Cbx_2(i,j)*( (fields->Hzx2(i,j)+fields->Hzy2(i,j) ) - (fields->Hzx2(i-1,j)+fields->Hzy2(i-1,j) ) );
			}

		// PML Area 3 
		for(i=0;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==0)
					fields->Ey3(i,j) = coef->Cax_3(i,j)*fields->Ey3(i,j) - coef->Cbx_3(i,j)*( (fields->Hzx3(i,j)+fields->Hzy3(i,j) ) - (fields->Hzx7(X_SIZE-1,j)+fields->Hzy7(X_SIZE-1,j) ) );
				else
					fields->Ey3(i,j) = coef->Cax_3(i,j)*fields->Ey3(i,j) - coef->Cbx_3(i,j)*( (fields->Hzx3(i,j)+fields->Hzy3(i,j) ) - (fields->Hzx3(i-1,j)+fields->Hzy3(i-1,j) ) );
			}

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Ey4(i,j) = coef->Cax_4(i,j)*fields->Ey4(i,j) - coef->Cbx_4(i,j)*( (fields->Hzx4(i,j)+fields->Hzy4(i,j) ) - (fields->Hzx4(i-1,j)+fields->Hzy4(i-1,j) ) );


		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Ey5(i,j) = coef->Ca_5(i,j)*fields->Ey5(i,j) - coef->Cb_5(i,j)*( (fields->Hzx5(i,j)+fields->Hzy5(i,j) ) - (fields->Hzx1(PML_DEPTH-1,j)+fields->Hzy1(PML_DEPTH-1,j) ) );
				else
					fields->Ey5(i,j) = coef->Ca_5(i,j)*fields->Ey5(i,j) - coef->Cb_5(i,j)*( (fields->Hzx5(i,j)+fields->Hzy5(i,j) ) - (fields->Hzx5(i-1,j)+fields->Hzy5(i-1,j) ) );
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==0)
					fields->Ey6(i,j) = coef->Ca_6(i,j)*fields->Ey6(i,j) - coef->Cb_6(i,j)*( (fields->Hzx6(i,j)+fields->Hzy6(i,j) ) - fields->Hz(X_SIZE-1,j) );
				else
					fields->Ey6(i,j) = coef->Ca_6(i,j)*fields->Ey6(i,j) - coef->Cb_6(i,j)*( (fields->Hzx6(i,j)+fields->Hzy6(i,j) ) - (fields->Hzx6(i-1,j)+fields->Hzy6(i-1,j) ) );
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Ey7(i,j) = coef->Ca_7(i,j)*fields->Ey7(i,j) - coef->Cb_7(i,j)*( (fields->Hzx7(i,j)+fields->Hzy7(i,j) ) - (fields->Hzx4(PML_DEPTH-1,j)+fields->Hzy4(PML_DEPTH-1,j) ) );
				else
					fields->Ey7(i,j) = coef->Ca_7(i,j)*fields->Ey7(i,j) - coef->Cb_7(i,j)*( (fields->Hzx7(i,j)+fields->Hzy7(i,j) ) - (fields->Hzx7(i-1,j)+fields->Hzy7(i-1,j) ) );
			}

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
				fields->Ey8(i,j) = coef->Ca_8(i,j)*fields->Ey8(i,j) - coef->Cb_8(i,j)*( (fields->Hzx8(i,j)+fields->Hzy8(i,j) ) - (fields->Hzx8(i-1,j)+fields->Hzy8(i-1,j) ) );
	}

	/*********** 
	 * TM mode *
	 ***********/

	if(strcmp(mode,"TM")==0)
	{
		/**************************
		 *    Ez, Ezx, Ezy field  *
		 **************************/

		//source = 10*exp(-pow((20-iteration)/2,2.0));
		source = sin(2*M_PI*iteration*dT*FREQ);
		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==X_SIZE-1 && j==Y_SIZE-1)
				{
					fields->Ez(i,j) = coef->Ca(i,j)*fields->Ez(i,j) + coef->Cb(i,j)*(fields->Hx(i,0) - fields->Hx(i,Y_SIZE-1) - fields->Hy6(0,j) + fields->Hy(X_SIZE-1,j));
					if(i==48 && j==510)
						fields->Ez(i,j) += source;
				}
				else if(i==X_SIZE-1)
				{
					fields->Ez(i,j) = coef->Ca(i,j)*fields->Ez(i,j) + coef->Cb(i,j)*(fields->Hx(i,j+1) - fields->Hx(i,j) - fields->Hy6(0,j) + fields->Hy(X_SIZE-1,j));
					if(i==48 && j==510)
						fields->Ez(i,j) += source;
				}
				else if(j==Y_SIZE-1)
				{
					fields->Ez(i,j) = coef->Ca(i,j)*fields->Ez(i,j) + coef->Cb(i,j)*(fields->Hx5(i,0) - fields->Hx(i,j) - fields->Hy(i+1,j) + fields->Hy(i,j));
					if(i==48 && j==510)
						fields->Ez(i,j) += source;
				}
				else
				{
					fields->Ez(i,j) = coef->Ca(i,j)*fields->Ez(i,j) + coef->Cb(i,j)*(fields->Hx(i,j+1) - fields->Hx(i,j) - fields->Hy(i+1,j) + fields->Hy(i,j));
					if(i==48 && j==510)
						fields->Ez(i,j) += source;
				}
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==PML_DEPTH-1)
				{
					fields->Ezx1(i,j) = coef->Cax_1(i,j)*fields->Ezx1(i,j) - coef->Cbx_1(i,j)*(fields->Hy1(1,j) - fields->Hy1(i,j));
					fields->Ezy1(i,j) = coef->Cay_1(i,j)*fields->Ezy1(i,j) + coef->Cby_1(i,j)*(fields->Hx5(0,j+1) - fields->Hx1(i,j));
				}
				else
				{
					fields->Ezx1(i,j) = coef->Cax_1(i,j)*fields->Ezx1(i,j) - coef->Cbx_1(i,j)*(fields->Hy1(i+1,j) - fields->Hy1(i,j));
					fields->Ezy1(i,j) = coef->Cay_1(i,j)*fields->Ezy1(i,j) + coef->Cby_1(i,j)*(fields->Hx1(i,j+1) - fields->Hx1(i,j));
				}
			}
	
		// PML Area 2 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				fields->Ezx2(i,j) = coef->Cax_2(i,j)*fields->Ezx2(i,j) - coef->Cbx_2(i,j)*(fields->Hy2(i+1,j) - fields->Hy2(i,j));
				fields->Ezy2(i,j) = coef->Cay_2(i,j)*fields->Ezy2(i,j) + coef->Cby_2(i,j)*(fields->Hx2(i,j+1) - fields->Hx2(i,j));
			}
	
		// PML Area 3 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=1;j<PML_DEPTH-1;j++)
			{
				if(j==PML_DEPTH-2)
				{
					fields->Ezx3(i,j) = coef->Cax_3(i,j)*fields->Ezx3(i,j) - coef->Cbx_3(i,j)*(fields->Hy3(i+1,j) - fields->Hy3(i,j));
					fields->Ezy3(i,j) = coef->Cay_3(i,j)*fields->Ezy3(i,j) + coef->Cby_3(i,j)*(fields->Hx6(i,0) - fields->Hx3(i,j));
				}
				else
				{
					fields->Ezx3(i,j) = coef->Cax_3(i,j)*fields->Ezx3(i,j) - coef->Cbx_3(i,j)*(fields->Hy3(i+1,j) - fields->Hy3(i,j));
					fields->Ezy3(i,j) = coef->Cay_3(i,j)*fields->Ezy3(i,j) + coef->Cby_3(i,j)*(fields->Hx3(i,j+1) - fields->Hx3(i,j));
				}
			}

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==PML_DEPTH-1 && j==PML_DEPTH-1)
				{
					fields->Ezx4(i,j) = coef->Cax_4(i,j)*fields->Ezx4(i,j) - coef->Cbx_4(i,j)*(fields->Hy7(0,j) - fields->Hy4(i,j));
					fields->Ezy4(i,j) = coef->Cay_4(i,j)*fields->Ezy4(i,j) + coef->Cby_4(i,j)*(fields->Hx8(i,0) - fields->Hx4(i,j));
				}
				else if(i==PML_DEPTH-1)
				{
					fields->Ezx4(i,j) = coef->Cax_4(i,j)*fields->Ezx4(i,j) - coef->Cbx_4(i,j)*(fields->Hy7(0,j) - fields->Hy4(i,j));
					fields->Ezy4(i,j) = coef->Cay_4(i,j)*fields->Ezy4(i,j) + coef->Cby_4(i,j)*(fields->Hx4(i,j+1) - fields->Hx4(i,j));
				}
				else if(j==PML_DEPTH-1)
				{
					fields->Ezx4(i,j) = coef->Cax_4(i,j)*fields->Ezx4(i,j) - coef->Cbx_4(i,j)*(fields->Hy4(i+1,j) - fields->Hy4(i,j));
					fields->Ezy4(i,j) = coef->Cay_4(i,j)*fields->Ezy4(i,j) + coef->Cby_4(i,j)*(fields->Hx8(i,0) - fields->Hx4(i,j));
				}
				else
				{
					fields->Ezx4(i,j) = coef->Cax_4(i,j)*fields->Ezx4(i,j) - coef->Cbx_4(i,j)*(fields->Hy4(i+1,j) - fields->Hy4(i,j));
					fields->Ezy4(i,j) = coef->Cay_4(i,j)*fields->Ezy4(i,j) + coef->Cby_4(i,j)*(fields->Hx4(i,j+1) - fields->Hx4(i,j));
				}
			}

		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==X_SIZE-1)
				{
					fields->Ezx5(i,j) = coef->Ca_5(i,j)*fields->Ezx5(i,j) - coef->Cb_5(i,j)*(fields->Hy2(0,j) - fields->Hy5(i,j));
					fields->Ezy5(i,j) = coef->Ca_5(i,j)*fields->Ezy5(i,j) + coef->Cb_5(i,j)*(fields->Hx5(i,j+1) - fields->Hx5(i,j));
				}
				else
				{
					fields->Ezx5(i,j) = coef->Ca_5(i,j)*fields->Ezx5(i,j) - coef->Cb_5(i,j)*(fields->Hy5(i+1,j) - fields->Hy5(i,j));
					fields->Ezy5(i,j) = coef->Ca_5(i,j)*fields->Ezy5(i,j) + coef->Cb_5(i,j)*(fields->Hx5(i,j+1) - fields->Hx5(i,j));
				}
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==Y_SIZE-1)
				{
					fields->Ezx6(i,j) = coef->Ca_6(i,j)*fields->Ezx6(i,j) - coef->Cb_6(i,j)*(fields->Hy6(i+1,j) - fields->Hy6(i,j));
					fields->Ezy6(i,j) = coef->Ca_6(i,j)*fields->Ezy6(i,j) + coef->Cb_6(i,j)*(fields->Hx2(i,0) - fields->Hx6(i,j));
				}
				else
				{
					fields->Ezx6(i,j) = coef->Ca_6(i,j)*fields->Ezx6(i,j) - coef->Cb_6(i,j)*(fields->Hy6(i+1,j) - fields->Hy6(i,j));
					fields->Ezy6(i,j) = coef->Ca_6(i,j)*fields->Ezy6(i,j) + coef->Cb_6(i,j)*(fields->Hx6(i,j+1) - fields->Hx6(i,j));
				}
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==X_SIZE-1 && j==PML_DEPTH-1)
				{
					fields->Ezx7(i,j) = coef->Ca_7(i,j)*fields->Ezx7(i,j) - coef->Cb_7(i,j)*(fields->Hy3(0,j) - fields->Hy7(i,j));
					fields->Ezy7(i,j) = coef->Ca_7(i,j)*fields->Ezy7(i,j) + coef->Cb_7(i,j)*(fields->Hx(i,0) - fields->Hx7(i,j));
				}
				else if(i==X_SIZE-1)
				{
					fields->Ezx7(i,j) = coef->Ca_7(i,j)*fields->Ezx7(i,j) - coef->Cb_7(i,j)*(fields->Hy3(0,j) - fields->Hy7(i,j));
					fields->Ezy7(i,j) = coef->Ca_7(i,j)*fields->Ezy7(i,j) + coef->Cb_7(i,j)*(fields->Hx7(i,j+1) - fields->Hx7(i,j));
				}
				else if(j==PML_DEPTH-1)
				{
					fields->Ezx7(i,j) = coef->Ca_7(i,j)*fields->Ezx7(i,j) - coef->Cb_7(i,j)*(fields->Hy7(i+1,j) - fields->Hy7(i,j));
					fields->Ezy7(i,j) = coef->Ca_7(i,j)*fields->Ezy7(i,j) + coef->Cb_7(i,j)*(fields->Hx(i,0) - fields->Hx7(i,j));
				}
				else
				{
					fields->Ezx7(i,j) = coef->Ca_7(i,j)*fields->Ezx7(i,j) - coef->Cb_7(i,j)*(fields->Hy7(i+1,j) - fields->Hy7(i,j));
					fields->Ezy7(i,j) = coef->Ca_7(i,j)*fields->Ezy7(i,j) + coef->Cb_7(i,j)*(fields->Hx7(i,j+1) - fields->Hx7(i,j));
				}
			}

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==PML_DEPTH-1 && j==Y_SIZE-1)
				{
					fields->Ezx8(i,j) = coef->Ca_8(i,j)*fields->Ezx8(i,j) - coef->Cb_8(i,j)*(fields->Hy(0,j) - fields->Hy8(i,j));
					fields->Ezy8(i,j) = coef->Ca_8(i,j)*fields->Ezy8(i,j) + coef->Cb_8(i,j)*(fields->Hx1(i,0) - fields->Hx8(i,j));
				}
				else if(i==PML_DEPTH-1)
				{
					fields->Ezx8(i,j) = coef->Ca_8(i,j)*fields->Ezx8(i,j) - coef->Cb_8(i,j)*(fields->Hy(0,j) - fields->Hy8(i,j));
					fields->Ezy8(i,j) = coef->Ca_8(i,j)*fields->Ezy8(i,j) + coef->Cb_8(i,j)*(fields->Hx8(i,j+1) - fields->Hx8(i,j));
				}
				else if(j==Y_SIZE-1)
				{
					fields->Ezx8(i,j) = coef->Ca_8(i,j)*fields->Ezx8(i,j) - coef->Cb_8(i,j)*(fields->Hy8(i+1,j) - fields->Hy8(i,j));
					fields->Ezy8(i,j) = coef->Ca_8(i,j)*fields->Ezy8(i,j) + coef->Cb_8(i,j)*(fields->Hx1(i,0) - fields->Hx8(i,j));
				}
				else
				{
					fields->Ezx8(i,j) = coef->Ca_8(i,j)*fields->Ezx8(i,j) - coef->Cb_8(i,j)*(fields->Hy8(i+1,j) - fields->Hy8(i,j));
					fields->Ezy8(i,j) = coef->Ca_8(i,j)*fields->Ezy8(i,j) + coef->Cb_8(i,j)*(fields->Hx8(i,j+1) - fields->Hx8(i,j));
				}
			}

		/**************************
		 *       Hx field         *
		 **************************/

		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Hx(i,j) = coef->Da_7(i,PML_DEPTH-1)*fields->Hx(i,j) + coef->Db_7(i,PML_DEPTH-1)*(fields->Ez(i,j) - ( fields->Ezx7(i,PML_DEPTH-1)+fields->Ezy7(i,PML_DEPTH-1) ));
				else
					fields->Hx(i,j) = coef->Da(i,j)*fields->Hx(i,j) + coef->Db(i,j)*(fields->Ez(i,j) - fields->Ez(i,j-1));
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Hx1(i,j) = coef->Day_1(i,j)*fields->Hx1(i,j) + coef->Dby_1(i,j)*( ( fields->Ezx1(i,j)+fields->Ezy1(i,j) )-(fields->Ezx8(i,Y_SIZE-1)+fields->Ezy8(i,Y_SIZE-1)) );
				else
					fields->Hx1(i,j) = coef->Day_1(i,j)*fields->Hx1(i,j) + coef->Dby_1(i,j)*( ( fields->Ezx1(i,j)+fields->Ezy1(i,j) )-(fields->Ezx1(i,j-1)+fields->Ezy1(i,j-1)) );
			}

		// PML Area 2 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Hx2(i,j) = coef->Day_2(i,j)*fields->Hx2(i,j) + coef->Dby_2(i,j)*( ( fields->Ezx2(i,j)+fields->Ezy2(i,j) )-(fields->Ezx6(i,Y_SIZE-1)+fields->Ezy6(i,Y_SIZE-1)) );
				else
					fields->Hx2(i,j) = coef->Day_2(i,j)*fields->Hx2(i,j) + coef->Dby_2(i,j)*( ( fields->Ezx2(i,j)+fields->Ezy2(i,j) )-(fields->Ezx2(i,j-1)+fields->Ezy2(i,j-1)) );
			}

		// PML Area 3 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Hx3(i,j) = coef->Day_3(i,j)*fields->Hx3(i,j) + coef->Dby_3(i,j)*( ( fields->Ezx3(i,j)+fields->Ezy3(i,j) )-(fields->Ezx3(i,j-1)+fields->Ezy3(i,j-1)) );

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Hx4(i,j) = coef->Day_4(i,j)*fields->Hx4(i,j) + coef->Dby_4(i,j)*( ( fields->Ezx4(i,j)+fields->Ezy4(i,j) )-(fields->Ezx4(i,j-1)+fields->Ezy4(i,j-1)) );

		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH;j++)
			{
				if(j==0)
					fields->Hx5(i,j) = coef->Da_5(i,j)*fields->Hx5(i,j) + coef->Db_5(i,j)*( ( fields->Ezx5(i,j)+fields->Ezy5(i,j) )-fields->Ez(i,Y_SIZE-1) );
				else
					fields->Hx5(i,j) = coef->Da_5(i,j)*fields->Hx5(i,j) + coef->Db_5(i,j)*( ( fields->Ezx5(i,j)+fields->Ezy5(i,j) )-(fields->Ezx5(i,j-1)+fields->Ezy5(i,j-1)) );
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH-1;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Hx6(i,j) = coef->Da_6(i,j)*fields->Hx6(i,j) + coef->Db_6(i,j)*( ( fields->Ezx6(i,j)+fields->Ezy6(i,j) )-(fields->Ezx3(i,PML_DEPTH-1)+fields->Ezy3(i,PML_DEPTH-1)) );
				else
					fields->Hx6(i,j) = coef->Da_6(i,j)*fields->Hx6(i,j) + coef->Db_6(i,j)*( ( fields->Ezx6(i,j)+fields->Ezy6(i,j) )-(fields->Ezx6(i,j-1)+fields->Ezy6(i,j-1)) );
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Hx7(i,j) = coef->Da_7(i,j)*fields->Hx7(i,j) + coef->Db_7(i,j)*( ( fields->Ezx7(i,j)+fields->Ezy7(i,j) )-(fields->Ezx7(i,j-1)+fields->Ezy7(i,j-1)) );

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(j==0)
					fields->Hx8(i,j) = coef->Da_8(i,j)*fields->Hx8(i,j) + coef->Db_8(i,j)*( ( fields->Ezx8(i,j)+fields->Ezy8(i,j) )-(fields->Ezx4(i,PML_DEPTH-1)+fields->Ezy4(i,PML_DEPTH-1)) );
				else
					fields->Hx8(i,j) = coef->Da_8(i,j)*fields->Hx8(i,j) + coef->Db_8(i,j)*( ( fields->Ezx8(i,j)+fields->Ezy8(i,j) )-(fields->Ezx8(i,j-1)+fields->Ezy8(i,j-1)) );
			}



		/**************************
		 *       Hy field         *
		 **************************/
	
		// Free space 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==0)
					fields->Hy(i,j) = coef->Da_8(PML_DEPTH-1,j)*fields->Hy(i,j) - coef->Db_8(PML_DEPTH-1,j)*( fields->Ez(i,j) - (fields->Ezx8(PML_DEPTH-1,j)+fields->Ezy8(PML_DEPTH-1,j) ));
				else
					fields->Hy(i,j) = coef->Da(i,j)*fields->Hy(i,j) - coef->Db(i,j)*(fields->Ez(i,j) - fields->Ez(i-1,j));
			}

		// PML Area 1 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
				fields->Hy1(i,j) = coef->Dax_1(i,j)*fields->Hy1(i,j) - coef->Dbx_1(i,j)*( (fields->Ezx1(i,j)+fields->Ezy1(i,j) ) - ( fields->Ezx1(i-1,j)+fields->Ezy1(i-1,j)) );

		// PML Area 2 
		for(i=0;i<PML_DEPTH;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Hy2(i,j) = coef->Dax_2(i,j)*fields->Hy2(i,j) - coef->Dbx_2(i,j)*( (fields->Ezx2(i,j)+fields->Ezy2(i,j) ) - (fields->Ezx5(X_SIZE-1,j)+fields->Ezy5(X_SIZE-1,j) ) );
				else
					fields->Hy2(i,j) = coef->Dax_2(i,j)*fields->Hy2(i,j) - coef->Dbx_2(i,j)*( (fields->Ezx2(i,j)+fields->Ezy2(i,j) ) - (fields->Ezx2(i-1,j)+fields->Ezy2(i-1,j) ) );
			}

		// PML Area 3 
		for(i=0;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
			{
				if(i==0)
					fields->Hy3(i,j) = coef->Dax_3(i,j)*fields->Hy3(i,j) - coef->Dbx_3(i,j)*( (fields->Ezx3(i,j)+fields->Ezy3(i,j) ) - (fields->Ezx7(X_SIZE-1,j)+fields->Ezy7(X_SIZE-1,j) ) );
				else
					fields->Hy3(i,j) = coef->Dax_3(i,j)*fields->Hy3(i,j) - coef->Dbx_3(i,j)*( (fields->Ezx3(i,j)+fields->Ezy3(i,j) ) - (fields->Ezx3(i-1,j)+fields->Ezy3(i-1,j) ) );
			}

		// PML Area 4 
		for(i=1;i<PML_DEPTH;i++)
			for(j=1;j<PML_DEPTH;j++)
				fields->Hy4(i,j) = coef->Dax_4(i,j)*fields->Hy4(i,j) - coef->Dbx_4(i,j)*( (fields->Ezx4(i,j)+fields->Ezy4(i,j) ) - (fields->Ezx4(i-1,j)+fields->Ezy4(i-1,j) ) );


		// PML Area 5 
		for(i=0;i<X_SIZE;i++)
			for(j=0;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Hy5(i,j) = coef->Da_5(i,j)*fields->Hy5(i,j) - coef->Db_5(i,j)*( (fields->Ezx5(i,j)+fields->Ezy5(i,j) ) - (fields->Ezx1(PML_DEPTH-1,j)+fields->Ezy1(PML_DEPTH-1,j) ) );
				else
					fields->Hy5(i,j) = coef->Da_5(i,j)*fields->Hy5(i,j) - coef->Db_5(i,j)*( (fields->Ezx5(i,j)+fields->Ezy5(i,j) ) - (fields->Ezx5(i-1,j)+fields->Ezy5(i-1,j) ) );
			}

		// PML Area 6 
		for(i=0;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
			{
				if(i==0)
					fields->Hy6(i,j) = coef->Da_6(i,j)*fields->Hy6(i,j) - coef->Db_6(i,j)*( (fields->Ezx6(i,j)+fields->Ezy6(i,j) ) - fields->Ez(X_SIZE-1,j) );
				else
					fields->Hy6(i,j) = coef->Da_6(i,j)*fields->Hy6(i,j) - coef->Db_6(i,j)*( (fields->Ezx6(i,j)+fields->Ezy6(i,j) ) - (fields->Ezx6(i-1,j)+fields->Ezy6(i-1,j) ) );
			}

		// PML Area 7 
		for(i=0;i<X_SIZE;i++)
			for(j=1;j<PML_DEPTH-1;j++)
			{
				if(i==0)
					fields->Hy7(i,j) = coef->Da_7(i,j)*fields->Hy7(i,j) - coef->Db_7(i,j)*( (fields->Ezx7(i,j)+fields->Ezy7(i,j) ) - (fields->Ezx4(PML_DEPTH-1,j)+fields->Ezy4(PML_DEPTH-1,j) ) );
				else
					fields->Hy7(i,j) = coef->Da_7(i,j)*fields->Hy7(i,j) - coef->Db_7(i,j)*( (fields->Ezx7(i,j)+fields->Ezy7(i,j) ) - (fields->Ezx7(i-1,j)+fields->Ezy7(i-1,j) ) );
			}

		// PML Area 8 
		for(i=1;i<PML_DEPTH;i++)
			for(j=0;j<Y_SIZE;j++)
				fields->Hy8(i,j) = coef->Da_8(i,j)*fields->Hy8(i,j) - coef->Db_8(i,j)*( (fields->Ezx8(i,j)+fields->Ezy8(i,j) ) - (fields->Ezx8(i-1,j)+fields->Ezy8(i-1,j) ) );
	}

}
